/* MAIN.C - start of MConfig */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#ifdef HAVE_MALLOC_H
# include <malloc.h>
#endif
#include <sys/time.h>
#include "../snmplib/asn1.h"
#include "../snmplib/snmp_api.h"
#include "util.h"
extern int App_Init __P((Tcl_Interp *interp));
extern const char tcl_mconfig[], tcl_lib[];

/*
 * Declarations for various library procedures and variables (don't want
 * to include tkInt.h or tkConfig.h here, because people might copy this
 * file out of the Tk source directory to make their own modified versions).
 */

/* extern void      exit _ANSI_ARGS_((int status)); */
extern int      isatty _ANSI_ARGS_((int fd));
extern int      read _ANSI_ARGS_((int fd, char *buf, size_t size));
extern char *      strrchr _ANSI_ARGS_((CONST char *string, int c));

/*
 * Global variables used by the main program:
 */
Tcl_Interp *tcl;
Tk_Window tk_main;   /* The main window for the application.  If
                      * NULL then the application no longer
                      * exists. */
#if 0
char *tcl_RcFileName = NULL;   /* Name of a user-specific startup script
                                * to source if the application is being run
                                * interactively (e.g. "~/.wishrc").  Set
                                * by Tcl_AppInit.  NULL means don't source
                                * anything ever. */
#endif
static int tty;         /* Non-zero means standard input is a
                         * terminal-like device.  Zero means it's
                         * a file. */
static char errorExitCmd[] = "exit 1";

/*
 * Command-line options:
 */
static int synchronize = 0;
static char *fileName = NULL;
static char *name = NULL;
static char *display = NULL;
static char *geometry = NULL;

static Tk_ArgvInfo argTable[] = {
    {"-file", TK_ARGV_STRING, (char *) NULL, (char *) &fileName,
   "File from which to read commands"},
    {"-geometry", TK_ARGV_STRING, (char *) NULL, (char *) &geometry,
   "Initial geometry for window"},
    {"-display", TK_ARGV_STRING, (char *) NULL, (char *) &display,
   "Display to use"},
    {"-name", TK_ARGV_STRING, (char *) NULL, (char *) &name,
   "Name to use for application"},
    {"-sync", TK_ARGV_CONSTANT, (char *) 1, (char *) &synchronize,
   "Use synchronous mode for display server"},
    {"-retries", TK_ARGV_INT, (char *) NULL, (char *) &retransmission,
   "Number of retries"},
    {"-timeout", TK_ARGV_INT, (char *) NULL, (char *) &timeout,
   "Time to wait for response"},
    {"-community", TK_ARGV_STRING, (char *) NULL, (char *) &community,
   "Community string"},
    {"-addronly", TK_ARGV_CONSTANT, (char *) 1, (char *) &addronly,
   "Display IP addresses in place of hostnames"},
    {(char *) NULL, TK_ARGV_END, (char *) NULL, (char *) NULL,
   (char *) NULL}
};

void
tcl_exec(str)
   const char str[];
{
   int code;
   int len = strlen(str)+1;
   char *ptr = (char *)malloc(len);

   strcpy(ptr, str);
   code = Tcl_Eval(tcl, ptr);
   free(ptr);

   if (code != TCL_OK) {
      ptr = Tcl_GetVar(tcl, "errorInfo", TCL_GLOBAL_ONLY);
      if (ptr == NULL)
         ptr = tcl->result;
      fprintf(stderr, "%s\n", ptr);
      Tcl_Eval(tcl, errorExitCmd);
   }
}

int
main(argc, argv)
   int argc;            /* Number of arguments. */
   char **argv;         /* Array of argument strings. */
{
   char *args, *p;
   char buf[20];
   int code;

   tcl = Tcl_CreateInterp();

   if (Tk_ParseArgv(tcl, (Tk_Window) NULL, &argc, argv, argTable, 0)!=TCL_OK) {
      fprintf(stderr, "%s\n", tcl->result);
      exit(1);
   }
   if (!name) {
      if (fileName)
         p = fileName;
      else
         p = argv[0];
      name = strrchr(p, '/');
      if (name)
         name++;
      else
         name = p;
   }

   /*
    * If a display was specified, put it into the DISPLAY
    * environment variable so that it will be available for
    * any sub-processes created by us.
    */
   if (display != NULL)
      Tcl_SetVar2(tcl, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);

   /*
    * Initialize the Tk application.
    */
   tk_main = Tk_CreateMainWindow(tcl, display, name, "mview");
   if (tk_main == NULL) {
      fprintf(stderr, "%s\n", tcl->result);
      exit(1);
   }
   if (synchronize)
      XSynchronize(Tk_Display(tk_main), True);
   Tk_GeometryRequest(tk_main, 200, 200);

   /*
    * Make command-line arguments available in the Tcl variables "argc"
    * and "argv".  Also set the "geometry" variable from the geometry
    * specified on the command line.
    */
   args = Tcl_Merge(argc-1, argv+1);
   Tcl_SetVar(tcl, "argv", args, TCL_GLOBAL_ONLY);
   ckfree(args);
   sprintf(buf, "%d", argc-1);
   Tcl_SetVar(tcl, "argc", buf, TCL_GLOBAL_ONLY);
   Tcl_SetVar(tcl, "argv0", (fileName) ? fileName : argv[0], TCL_GLOBAL_ONLY);
   if (geometry)
      Tcl_SetVar(tcl, "geometry", geometry, TCL_GLOBAL_ONLY);

   /*
    * Set the "tcl_interactive" variable.
    */
   tty = isatty(0);
   Tcl_SetVar(tcl, "tcl_interactive",
      (!fileName && tty) ? "1" : "0", TCL_GLOBAL_ONLY);

   /*
    * Invoke application-specific initialization.
    */
   if (Tcl_AppInit(tcl) != TCL_OK)
      fprintf(stderr, "Tcl_AppInit failed: %s\n", tcl->result);

   /*
    * Set the geometry of the main window, if requested.
    */
   if (geometry != NULL) {
      code = Tcl_VarEval(tcl, "wm geometry . ", geometry, (char *) NULL);
      if (code != TCL_OK)
         fprintf(stderr, "%s\n", tcl->result);
   }

   /*
    * Invoke the startup script 
    */
   tcl_exec(tcl_lib);
   tcl_exec(tcl_mconfig);

   /*
    * Loop infinitely, waiting for commands to execute.  When there
    * are no windows left, Tk_MainLoop returns and we exit.
    */
   Tk_MainLoop();

   /*
    * Don't exit directly, but rather invoke the Tcl "exit" command.
    * This gives the application the opportunity to redefine "exit"
    * to do additional cleanup.
    */
   Tcl_Eval(tcl, "exit");
   exit(1);
}

int
Tcl_AppInit(interp)
   Tcl_Interp *interp;		/* Interpreter for application. */
{
/*
   if (Tcl_Init(interp) == TCL_ERROR)
      return TCL_ERROR;
   if (Tk_Init(interp) == TCL_ERROR)
      return TCL_ERROR;
*/
   if (App_Init(interp) == TCL_ERROR)
      return TCL_ERROR;
   tcl_RcFileName = ".tclrc";
   return TCL_OK;
}
